import axios from 'axios'
import Util from '@/libs/util'
import router from '@/router/index'
import Setting from '@/setting'
import store from '@/store'
import api from '@/api'
import { Message } from 'element-ui'

const baseURL = Setting.apiBaseURL
const service = axios.create({
  baseURL,
  timeout: 10000000
})
// post请求头
service.defaults.headers.post['Content-Type'] = 'application/json;charset=UTF-8'

let check = true
let checkTimer = null
// 检测账号是否被禁用，如果禁用则提示并退出登录
function checkDisabled () {
  const token = Util.local.get(Setting.tokenKey)
  if (!token) return false
  check = false
  axios.get(baseURL + api.accountIsDisabled + '?token=' + token, {
    token: Util.local.get(Setting.tokenKey),
    headers: {
      token
    }
  }).then(res => {
    if (res.data.code === 401) {
      Util.local.remove(Setting.storeKey)
      Util.local.remove(Setting.tokenKey)
      Util.errorMsg(res.data.msg.includes('顶') ? '您的账号已在其他设备登录，您已被迫下线！' : '登录过期，请重新登录！')
      setTimeout(() => {
        location.reload()
      }, 1500)
    } else if (res.data) {
      Util.errorMsg('账号被禁用，已强制退出，有疑问请联系管理员')
      setTimeout(() => {
        store.dispatch('user/logout')
      }, 1500)
    }
  }).catch(res => { })
}
// 请求拦截器
service.interceptors.request.use(config => {
  let token = Util.local.get(Setting.tokenKey)
  if (token) config.headers.token = token
  let storeSchoolId = store.state.user.schoolId
  let routeSchoolId = router.app._route.query.schoolId
  if (routeSchoolId) routeSchoolId = atob(decodeURI(routeSchoolId))
  // 如果store里没有schoolId,则取url里的schoolId,从用户跳转到后台的时候因为加载顺序的问题会出现这个情况,所以要做这个处理
  if (storeSchoolId || routeSchoolId) config.headers.schoolId = storeSchoolId ? storeSchoolId : routeSchoolId
  clearTimeout(checkTimer)
  check && checkDisabled()
  checkTimer = setTimeout(() => {
    check = true
  }, 2000)
  return config
}, err => {
  Util.errorMsg({
    message: '退出登陆',
    onClose: function () {
      router.push({ name: '/login' })
    }
  })
  return Promise.reject(err)
})

// 响应拦截器
service.interceptors.response.use(
  response => {
    const res = response.data
    if (res.status == 200) {
      return Promise.resolve(res).catch(e => { })
    } else if (res.code === 401) {
      // 账号互踢
      if (!logouted) {
        Util.local.remove(Setting.storeKey)
        Util.local.remove(Setting.tokenKey)
        Util.errorMsg(res.msg.includes('顶') ? '您的账号已在其他设备登录，您已被迫下线！' : '登录过期，请重新登录！')
        setTimeout(() => {
          router.replace({
            path: '/login'
          })
          logouted = 0
        }, 1500)
        logouted = 1
      }
    } else if (res.status == 30001) {
      return Promise.resolve(res).catch(e => { })
    } else if (!res.status) {
      return Promise.resolve(res).catch(e => { })
    } else if (res.status === 10008) {
      Message.error({
        message: res.message,
        showClose: true,
        duration: 1500
      })
      return Promise.reject(res)
    } else {
      Util.errorMsg(res.message)
      res.message === '该账号无权限！' && setTimeout(() => {
        store.dispatch('user/logout')
      }, 1500)
      return Promise.reject(res)
    }
  },
  // 服务器状态码不是200的情况
  error => {
    if (error.response.status) {
      switch (error.response.status) {
        // 401: 未登录
        // 未登录则跳转登录页面，并携带当前页面的路径
        // 在登录成功后返回当前页面，这一步需要在登录页操作。
        case 401:
          Util.local.remove(Setting.storeKey)
          Util.local.remove(Setting.tokenKey)
          Util.errorMsg('登录过期，请重新登录')
          setTimeout(() => {
            router.replace({
              path: '/login'
            })
          }, 1000)
          break
        case 500:
          Util.errorMsg('网络错误')
          break
        // 403 token过期
        // 登录过期对用户进行提示
        // 清除本地token和清空vuex中token对象
        // 跳转登录页面
        case 403:
          Util.local.remove(Setting.storeKey)
          Util.local.remove(Setting.tokenKey)
          Util.errorMsg('登录过期，请重新登录')
          // 清除token
          // store.commit('loginSuccess', null);
          // 跳转登录页面，并将要浏览的页面fullPath传过去，登录成功后跳转需要访问的页面
          setTimeout(() => {
            router.replace({
              path: '/login',
              // query: {
              //     redirect: router.currentRoute.fullPath
              // }
            })
          }, 1000)
          break
        // 404请求不存在
        case 404:
          Util.errorMsg('网络请求不存在!')
          break
        // 其他错误，直接抛出错误提示
        default:
          Util.errorMsg(error.response.data.message)
          Promise.reject(res)
      }
      return Promise.reject(error.response)
    }
  }
);

function get (url, params) {
  return new Promise((resolve, reject) => {
    service.get(url, { params: params }).then(res => {
      resolve(res)
    }).catch(err => {
      reject(err)
    })
  })
}

function post (url, params) {
  return new Promise((resolve, reject) => {
    service.post(url, params).then(res => {
      resolve(res)
    }).catch(err => {
      reject(err.data)
    })
  })
}

function del (url, params) {
  return new Promise((resolve, reject) => {
    service.delete(url, {
      params
    }).then(res => {
      resolve(res)
    }).catch(err => {
      reject(err.data)
    })
  })
}

function put (url, params) {
  return new Promise((resolve, reject) => {
    service.put(url, {
      params
    }).then(res => {
      resolve(res)
    }).catch(err => {
      reject(err.data)
    })
  })
}
export { get, post, del, put }